tinymce.confluence.formatter = new (function ($) {
    /**
     * This structure describes how to create a new instance of a block format.
     * elements: an array of descriptions of elements which contain the content, from the inner element to the outer element
     * rawText: if true, extract the raw text from the macro body and throw any formatting away.
     * innerSelector: a jquery selector used to locate the inner most element
     */
    var formatDescriptors = {
        p: {elements:[{name:"p"}], rawText:true},
        h1: {elements:[{name:"h1"}], rawText:true},
        h2: {elements:[{name:"h2"}], rawText:true},
        h3: {elements:[{name:"h3"}], rawText:true},
        h4: {elements:[{name:"h4"}], rawText:true},
        h5: {elements:[{name:"h5"}], rawText:true},
        h6: {elements:[{name:"h6"}], rawText:true},
        macro_quote: {
            elements:[
                {name:"blockquote"},
                {name:"div", attributes:{"class" : "macro", "macrotext" : "{quote}", "command" : "quote"}}
            ],
            rawText:false,
            innerSelector: "blockquote" },
        macro_noformat: {
            elements:[
                {name:"pre"},
                {name:"div", attributes:{"class" : "preformattedContent panelContent"}},
                {name:"div", attributes:{"class" : "preformatted panel"}},
                {name:"div", attributes:{"class" : "macro", "macrotext":"{noformat}", "command" : "noformat"}}
            ],
            rawText:true,
            innerSelector: "div.preformatted div.preformattedContent pre" },
        macro_panel: {
            elements:[
                {name:"div", attributes:{"class" : "panelContent"}},
                {name:"div", attributes:{"class" : "panel"}},
                {name:"div", attributes:{"class" : "macro", "macrotext" : "{panel}", "command" : "panel"}}
            ],
            rawText:false,
            innerSelector: "div.panel div.panelContent" },
        macro_code: {
            elements:[
                {name:"pre", attributes:{"class" : "code-java"}},
                {name:"div", attributes:{"class" : "codeContent panelContent"}},
                {name:"div", attributes:{"class" : "code panel"}},
                {name:"div", attributes:{"class" : "macro", "macrotext" : "{code}", "command" : "code"}}
            ],
            rawText:true,
            innerSelector: "div.code div.codeContent pre.code-java" }
    };

    // Public (privileged) methods here - called from tiny_mce_src.js
    this.formatBlock = function(format) {
        var ed = tinyMCE.activeEditor, selection = ed.selection, domUtils = ed.dom;
        var outerBlock = getEnclosingBlock(selection.getNode());
        console.log("formatBlock outerBlock=" + outerBlock);

        // get the bookmark before we do any formatting
        var bookmark = selection.getBookmark();

        if (!format) {
            removeFormat(outerBlock, domUtils);
        }
        else if (!outerBlock) {
            formatRange(ed, ed.selection.getRng(), domUtils, format);
        }
        else {
            formatElement(outerBlock, domUtils, format);
        }

        // clean up code
        selection.moveToBookmark(bookmark);
        ed.nodeChanged();

        // only execute the FormatBlock command for known formatting
        if(/^(P|H[1-6])$/i.test(format))
            ed.getDoc().execCommand('FormatBlock', false, format);
    };

    // Private members - variables and functions.
    function formatElement(outerBlock, domUtils, format) {
        console.log("formatElement");
        // find the current block element we are about to apply formatting to
        var $blockToFormat = $(outerBlock);
        if (isMacroBlock(outerBlock))
            $blockToFormat = findMacroBody(outerBlock);

        var formatDescriptor = formatDescriptors[format];
        var $newFormatBlock = $(createHtmlElement(formatDescriptor, domUtils));
        var $placeToInsert = $newFormatBlock;

        if (formatDescriptor.innerSelector) {
            $placeToInsert = $newFormatBlock.find(formatDescriptor.innerSelector);
        }

        if (formatDescriptor.rawText) {
            var text = $blockToFormat.text();
            text = text || " ";
            $placeToInsert.text(text);
            domUtils.replace($newFormatBlock[0], outerBlock, false);
        }
        else { // move children in enclosingBlock to newFormatBlock
            $placeToInsert.append($blockToFormat[0].childNodes);
            domUtils.replace($newFormatBlock[0], outerBlock, false);
        }
        console.log("$newFormatBlock.text()=" + $newFormatBlock.text());
    };

    function formatRange(editor, range, domUtils, format) {
        console.log("formatRange");
        var formatDescriptor = formatDescriptors[format];
        var $newFormatBlock = $(createHtmlElement(formatDescriptor, domUtils));
        var $placeToInsert = $newFormatBlock;

        if (formatDescriptor.innerSelector) {
            $placeToInsert = $newFormatBlock.find(formatDescriptor.innerSelector);
        }

        var rangeFragment = range.extractContents ? range.extractContents() : range;

        if (formatDescriptor.rawText) {
            if (range.text) {
                $placeToInsert.html(range.text);
            }
            else {
                $placeToInsert.append(rangeFragment.childNodes);
                $placeToInsert.html($placeToInsert.text());
            }
        }
        else {
            if (range.htmlText)
                $placeToInsert.html(range.htmlText);
            else
                $placeToInsert.append(rangeFragment.childNodes);
        }
        // TODO - this method is flaky in Safari, the $newFormatBlock[0] has the right html content.
        editor.selection.setNode($newFormatBlock[0]);
    };

    function createHtmlElement(formatDescriptor, domUtils) {
        var newElement;
        $.each(formatDescriptor.elements, function (i, element) {
            var createdNode = domUtils.create(element.name, element.attributes);
            if (newElement)
                createdNode.appendChild(newElement);

            newElement = createdNode;
        });
        return newElement;
    };

    /**
     * Returns a html element that is an 'enclosing block' of the given node. An enclosing block in this context
     * refers to a format block described in formatDescriptors.
     */
    function getEnclosingBlock (node) {
        // look for an enclosing macro first
        var $macroDiv = $(node).parents("div.macro");
        if ($macroDiv.length)
            return $macroDiv[0];

        if (/^(P|DIV|H[1-6]|PRE)$/i.test(node.nodeName))
            return node;

        // otherwise we try and find the first ancestor that is a block element
        var enclosingElem = null;
        $(node).parents().each(function(i, parent) {
            if (/^(P|DIV|H[1-6]|PRE)$/i.test(parent.nodeName)) {
                enclosingElem = parent;
                return false;
            }
        });
        return enclosingElem;
    };

    /**
     * Returns true if the given node is a macro block.
     */
    function isMacroBlock(node) {
        return $(node).attr("command");
    };

    /**
     * Returns a jquery object pointing to the innermost element holding the macro body.
     */
    function findMacroBody (node) {
        $node = $(node);
        var command = $node.attr("command");
        var descriptor = formatDescriptors["macro_" + command];
        if (descriptor) {
            var $macroBody = $node.find(descriptor.innerSelector);
            if($macroBody.length > 0)
                return $macroBody;

            console.log("ERROR: couldn't find macro body for '" + command + "'");
            return $node;
        }
        console.log("ERROR: no macro descriptor found for '" + command + "'");
    };

    function removeFormat(enclosingBlock, domUtils) {
        console.log("removeFormat");
        // Can't remove format from range - doesn't make sense.
        if (!enclosingBlock) {
            console.log("removeFormat: enclosingBlock null, returning");
            return;
        }

        var innerEnclosingBlock = enclosingBlock;
        if (isMacroBlock(enclosingBlock)) {
            innerEnclosingBlock = findMacroBody(enclosingBlock)[0];
        }

        console.log("enclosingBlock.parentNode.nodeName=" + enclosingBlock.parentNode.nodeName);
        if (enclosingBlock.parentNode.nodeName.toLowerCase() != "body") {
            $(enclosingBlock).replaceWith(innerEnclosingBlock.childNodes);
        }
        else {
            // Text in the editor should never be directly inside the body - should at least
            // be in a p. This is how the wiki-renderer does it too.
            var p =  domUtils.create("p");
            $(p).append(innerEnclosingBlock.childNodes);
            $(enclosingBlock).replaceWith(p);
        }
    };
})(AJS.$);
