/**
 * Implement the methods required by the com.atlassian.confluence.plugin.editor.Editor interface, adapting to a
 * TinyMCE editor implementation.
 *
 * Note that tinyMCE should not be used as it gets confused with the tinymce object. Use tinymce.EditorManager if that's
 * what you need.
 */
AJS.Editor.Adapter = {

    /**
     * Public interface methods on Editor.java
     */

    // called just after the DIV containing the editor is made visible
    onShowEditor : function () { },

    // called just before the DIV containing the editor is hidden
    onHideEditor : function () { },

    // put the text in newValue into the editor. This is called when the editor needs new
    // content -- it is *not* called to set the initial content. That should be done either by providing the
    // editor with the content as part of the initial HTML, or by calling javascript from editorOnLoad()
    setEditorValue : function (newValue) {
        if (newValue) {
            this.getEditor().setContent(newValue);
        }
    },

    /*
     *  If TinyMCE has not finished loading the page content, and the user switches tabs (from rich text to markup),
     *  block the switch, otherwise their content will be lost. (CONF-4824)
     */
    // return true if the editor is in a state where changes from rich text to markup and vice versa are allowed
    allowModeChange : function () {
        return this._tinyMceHasInit;
    },

    // return the current HTML contents of the editor. This *must* return a JavaScript string,
    // not a JavaObject wrapping a java.lang.String
    getEditorHTML : function () {
        return "" + this.getEditor().getContent();
    },

    // called in the page's onLoad handler, place any initialization needed at this point here
    editorOnLoad : function () { },

    // return true if the contents of the editor has been modified by the user since
    // the last time editorResetContentChanged()
    editorHasContentChanged : function () {
        return this.getEditor().isDirty();
    },

    // called to reset the contents change indicator
    editorResetContentChanged : function () {
        this.getEditor().setDirty(false);
    },

    /**
     * Non interface methods & variables
     */
    _tinyMceHasInit : false,

    getTinyMceHasInit: function () {
        return this._tinyMceHasInit;
    },

    getEditor : function () {
        return tinyMCE.activeEditor;
    },

    tinyMceOnInit : function() {
        console.log("Adapter:tinyMceOnInit oninit callback");
        this._tinyMceHasInit = true;

        if (this.onInitCallback) {
            this.onInitCallback();
        }
    },

    tinyMceEventHandler : function(e) {

        var ed = this.getEditor();

        // handle tabbing in lists
        if (e.keyCode == 9 && (ed.queryCommandState('InsertUnorderedList') || ed.queryCommandState('InsertOrderedList'))) {

            if (e.type == "keydown") {

                if(e.shiftKey) {
                    ed.execCommand("Outdent");
                }
                else {
                    ed.execCommand("Indent");
                }
                return false;
            }
            // stop firefox's default tab behaviour
            if (tinymce.isGecko && e.type == "keypress") {
                return AJS.stopEvent(e);
            }
        }

        return true; // otherwise continue with default handling
    },

    i18nCallback : function(response) {
        console.log("i18nCallback");

        for (var key in response) {
            tinymce.EditorManager.addI18n(key, response[key]);
        }
    },

    initialiseTinyMce : function() {
        var t = AJS.Editor.Adapter;
        tinymce.EditorManager.preInit.apply(tinymce.EditorManager);
        tinymce.EditorManager.init(t.settings);

        console.log("Adapter:after editor manager init");
        tinyMCE.onBeforeUnload.addToTop(AJS.Editor.handleUnload);
    },

    webResourcePath : "/download/resources/com.atlassian.confluence.tinymceplugin%3Atinymceeditor/",

    // gets the base url from the current location,
    getCurrentBaseUrl : function() {
        if (!this.currentBaseUrl) {
            var l = document.location;
            this.currentBaseUrl = l.protocol + "//" + l.hostname + (l.port ? ":" + l.port : "");
        }
        return this.currentBaseUrl;
    },

    // gets the static resource url prefix, which will include the caching headers
    getResourceUrlPrefix : function() {
        if (!this.resourceUrlPrefix) {
            this.resourceUrlPrefix = this.getCurrentBaseUrl() + AJS.params.editorPluginResourcePrefix;
        }
        return this.resourceUrlPrefix;
    },

    // gets the absolute url path to the tinymce web resources, which will include the caching headers
    getTinyMceBaseUrl : function() {
        if (!this.absoluteUrl) {
            this.absoluteUrl =  this.getResourceUrlPrefix() + this.webResourcePath + "tinymcesource/";
        }
        return this.absoluteUrl;
    }
};

AJS.toInit(function () {

    AJS.Editor.Adapter.settings = {

        // general
        width: "100%",
        height: AJS.params.paramsHeight,
        document_base_url: AJS.Editor.Adapter.getTinyMceBaseUrl(),
        language: AJS.params.actionLocale, 
        button_tile_map: true,
        auto_reset_designmode: true, // recommended for tabs and hidding editors
        plugins: "table,paste,emotions,fullscreen,confluence,spellchecker,searchreplace,iespell",

        // advanced theme params
        theme: "advanced",
        theme_advanced_buttons1: "formatselect,bold,italic,underline,strikethrough,forecolor,separator,table,row_before,row_after,delete_row,col_before,col_after,delete_col,separator,bullist,numlist,outdent,indent,separator,undo,redo,separator,hr,charmap,emotions,conflink,confimage,separator,fullscreen",
        theme_advanced_buttons2: "",
        theme_advanced_buttons3: "",
        theme_advanced_toolbar_location: "top",
        theme_advanced_toolbar_align: "left",
        theme_advanced_resizing: true,
        theme_advanced_resize_horizontal: false,
        theme_advanced_statusbar_location: "bottom",
        theme_advanced_path: false,
        theme_advanced_blockformats: "h1,h2,h3,h4,h5,h6,macro_quote,macro_panel,macro_code,macro_noformat",

        // selectors for tinymce editors
        mode: "textareas",
        editor_selector: "tinymce-editor",
        elements: "wysiwygTextarea", // editor id

        // callbacks
        oninit: "AJS.Editor.Adapter.tinyMceOnInit",
        handle_event_callback: "AJS.Editor.Adapter.tinyMceEventHandler",

        // table settings
        visual: false,
        confluence_table_style: "confluenceTable",
        confluence_table_cell_style: "confluenceTd",
        confluence_table_header_style: "confluenceTh",
        confluence_table_default_rows: 2,
        confluence_table_default_cols: 2,
        confluence_table_default_heading: true,

        // output settings
        cleanup: false, // don't clean up cause we have special macro attributes
        force_p_newlines: true,
        force_br_newlines: false,

        // layout settings
        body_class: "wiki-content",
        content_css: AJS.Editor.Adapter.getResourceUrlPrefix() + "/styles/combined.css?forWysiwyg=true&spaceKey=" + encodeURI(AJS.params.spaceKey),
        popup_css: AJS.Editor.Adapter.getResourceUrlPrefix() + "/styles/combined.css?spaceKey=" + encodeURI(AJS.params.spaceKey),

        resource_prefix: AJS.Editor.Adapter.getResourceUrlPrefix(),

        // confluence-specific settings
        context_path: AJS.params.contextPath,
        plugin_action_base_path: AJS.params.contextPath + "/plugins/tinymce",
        page_id: (AJS.params.pageId == 0 ? null : AJS.params.pageId),
        draft_type: (AJS.params.pageId == 0 ? AJS.params.draftType : null),
        form_name: AJS.params.formName,
        space_key: encodeURI(AJS.params.spaceKey),
        confluence_popup_width: 620,
        confluence_popup_height: 550
    };

    // request i18n for tinymce, then initialise tinymce
    AJS.$.ajax( { url: AJS.params.editorPluginResourcePrefix + "/plugins/servlet/language/" + AJS.params.actionLocale,
                  cache: true,
                  dataType: "json",
                  success: AJS.Editor.Adapter.i18nCallback,
                  complete: AJS.Editor.Adapter.initialiseTinyMce });
});
